/*
  Dynamic Focus Handler
  Developed on 2017 10 11
  Author: Michael Pierpaolini care of Datalogic S.p.A.
	
Ver. 1.0 :
 - Addition of a 'Last Acquisition' section to the GUI.
 - Trigger button has been added; by pressing this button, the script is simply asking the sensor to return the distance.
 - Addition of a second distance acquisition once that the distance has been reached after pressing the 'START AUTOFOCUS' button
 - External START is handled just as a 'Start Marking' event generated by pressing the button on the GUI
 
 Ver. 1.1:
 -Checkbox for marking after autofocus has been readapted;
 -EXTERNAL START is handled in the same way as the Start Autofocus button
 -Controlling that Lighter is running with an up to date version, which allows to use the scripting functionalities to control the Distance Sensor
 
 Ver. 1.2:
 -onQueryStart will handle debaouncing of the START signal, or multiple START signal arriving to the Laser marker during Autofocus or Marking process; events triggered during these moments, will be ignored
*/

/***********************************************************
  Definitions of constants
***********************************************************/
// Works definition

// Internal messages
const AppTitle = "Datalogic Automation Standard Serial Protocol Manager";
const SEL_ERROR = "Unable to select the requested work";
const LOAD_ERROR = "Unable to load the work set!";
const ESC_ERROR = "Invalid command detected";
const ESC_WARNING = "Unsupported command detected";
const STR_ERROR = "Unable to set the requested string data";
const CNT_ERROR = "Unable to get the requested counter";
const BUSY_ERROR = "Command received while system busy!";

const savedPosition=undefined;
//Tracing
const TRACE = true;
const DISTANCE = "DISTANCE";

const autofocusExecuting=false;

var M, m, measureHeight=false, setUpSensorPhase=true, step=0;

/***********************************************************
  Main function
***********************************************************/
function main()
{
  System.sigQueryStart.connect(onQueryStart);
  System.sigLaserStop.connect(onLaserStop);
  System.sigLaserStart.connect(onLaserStart);
  System.sigLaserEnd.connect(onLaserEnd);
  Axis.sigFocusReached.connect(onPositionReached);
  System["sigLaserError(int)"].connect(onLaserError);
  System.sigClose.connect(onClose);
  
  h_ver = System.version().split(".");
  print("Lighter version: "+h_ver);
  print("Is Lighter version compatible? "+checkLighterVersion(h_ver));
  hDoc = new LaserDoc();
  h_port = new classPort();
  h_Window = new classMainWindow(h_port);
  IoPort.setReady(true);
  
  m=Axis.getMinPosition(2);
  M=Axis.getMaxPosition(2);
  log("Z Axis enabled: "+Axis.isEnabled(2)+", Max: "+M+", Min: "+m);

  v=Axis.getSensorDistanceReference();
  if(v==-1){
      print("No distance reference value found on this laser");
      h_Window.PBfocusStart.enable=false;
      log("ERROR - Distance Sensor Status: "+Axis.getDistanceSensorStatus());
  }
  else {
      print("The reference distance is present on this laser: "+System.getStringValue(DISTANCE));
      h_Window.PBfocusStart.enable=true;
  }
  
  if (!h_Window.checkErrors()) print("NO ERRORS ARE PRESENT");
  else print("ERRORS ARE PRESENT");
  
  trace( "Std RS-232 Ready !" );
  h_Window.onTrigger();
  h_Window.onIndexChanged();

  log("Distance Sensor Status: "+Axis.getDistanceSensorStatus());
}

/***********************************************************
  Laser management functions
***********************************************************/
function onQueryStart()
{
  trace("onQueryStart");
  if(!autofocusExecuting) {
       h_Window.prv.preview(hDoc);
       h_Window.onFocusStart();
  }
  else {
      log("Ext. START cannot be processed as the Autofocus or Marking process is undergoing");
  }
 
}

function onMark() {
    print("onMark()");
    autofocusExecuting=true;
    h_Window.PBtrigger.enable=false;
    h_Window.PBsaveDistance.enable=false;
    h_Window.PBfocusStart.enable=false;
    hDoc.execute();
}

function onLaserStop()
{
  trace("onLaserStop");
}

function onLaserStart()
{
  // TODO
}

function onLaserEnd()
{
    autofocusExecuting=false;
    h_Window.PBtrigger.enable=true;
    h_Window.PBsaveDistance.enable=true;
    h_Window.PBfocusStart.enable=true;
}

/*START ADDITIONAL CODE part 4 - Implementation task #78612*/
function onPositionReached(result)
{
    print("onPositionReached - Axis.getDistanceSensorStatus(): "+Axis.getDistanceSensorStatus());
    if (result) {
	log("Focus Distance reached");
	h_Window.OK();
	if(h_Window.cbAutoManual.checked) onMark();
	else autofocusExecuting=false;
    }
    else{
	autofocusExecuting=false;
	log("Focus Distance cannot be reached");
	h_Window.checkErrors();
	print("Get Distance Sensor Status: "+Axis.getDistanceSensorStatus());
    }
    
}
/*END ADDITIONAL CODE part 4 - Implementation task #78612*/

function onLaserError(error)
{
  switch(error)
  {
    case System.DSP_IN_HANG:
      System.resetBoard();
      break;
    case System.DSP_ERROR_INIT:
      // DSp initialization failure
      break;
    // TODO
  }
}

function onClose()
{
  IoPort.setReady(false); 
  h_Window.close();
}
